// Pixel Profile
// efg, September 1998
// www.efg2.com/lab

unit gLProfileFrm;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs, ShellAPI,
  StdCtrls, ExtDlgs, ExtCtrls, ComCtrls, Buttons,
  gULineLibrary, // RestrictCursorToDrawingArea, TLineSelected, NearLine
  Grids, Spin, TeEngine, Series, TeeProcs, Chart;

type
  TDrawingTool = (dtNone, dtSelectTool, dtLineTool);

  TLineProfileForm = class(TForm)
    PageControlProfile: TPageControl;
    TabSheetData: TTabSheet;
    TabSheetGraphs: TTabSheet;
    ChartPixelProfile: TChart;
    SeriesRed: TLineSeries;
    SeriesGreen: TLineSeries;
    SeriesBlue: TLineSeries;
    SeriesIntensity: TLineSeries;
    CheckBoxRed: TCheckBox;
    CheckBoxGreen: TCheckBox;
    CheckBoxBlue: TCheckBox;
    CheckBoxIntensity: TCheckBox;
    CheckBoxHue: TCheckBox;
    CheckBoxSaturation: TCheckBox;
    CheckBoxValue: TCheckBox;
    SeriesHue: TLineSeries;
    SeriesSaturation: TLineSeries;
    SeriesValue: TLineSeries;
    ImagePixelRow: TImage;
    LabelPixelSlice: TLabel;
    LabelFilename: TLabel;
    GroupBoxDraw: TGroupBox;
    SpeedButtonLine: TSpeedButton;
    SpeedButtonSelect: TSpeedButton;
    CloseBtn: TSpeedButton;
    HelpBtn: TSpeedButton;
    LabelPoints: TLabel;
    efgLabel: TLabel;
    MemoData: TRichEdit;
    SaveButton: TSpeedButton;
    PrintButton: TSpeedButton;
    CopyButton: TSpeedButton;
    ButtonPrintGraph: TSpeedButton;
    SpeedButtonErase: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure StartProfile;
    procedure DoImageMouseDown(X, Y: Integer);
    procedure DoImageMouseMove(X, Y: Integer);
    procedure DoImageMouseUp(X, Y: Integer);

    procedure SpeedButtonEraseClick(Sender: TObject);
    procedure SpeedButtonToolClick(Sender: TObject);
    procedure CheckBoxPlaneClick(Sender: TObject);
    procedure ButtonPrintGraphClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure efgLabelClick(Sender: TObject);
    procedure HelpBtnClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure SaveButtonClick(Sender: TObject);
    procedure PrintButtonClick(Sender: TObject);
    procedure CopyButtonClick(Sender: TObject);

  private
{    Bitmap           :  TBitmap; }
{    ImageDesignHeight:  INTEGER;
    ImageDesignWidth :  INTEGER;}
{    Drawing          :  BOOLEAN;}
    DrawingTool: TDrawingTool;
    Selected: TLineSelected;
    ULPEndPoint: TPoint;
    ULPStartPoint: TPoint;
    MovingPoint: TPoint; // Point used when moving, stretching

    procedure CreateProfileLine;
    procedure UpdateImage;
    procedure UpdateLineLength;

  public
    { Public declarations }
  end;

var
  LineProfileForm: TLineProfileForm;

implementation
{$R *.DFM}

uses
  Math, // MinIntValue, MaxIntValue
  gUEnabler, gGlobal, gGrip;

const
  MaxPixelCount = 65536;
  RubberBandColor: TColor = clNavy;
  SquareHalfSize = 4;
  OffScreenCoordinate = -3 * SquareHalfSize;

type
  EImageProcessingError = class(Exception);
  TRGBTripleArray = array[0..MaxPixelCount - 1] of TRGBTriple;
  pRGBTripleArray = ^TRGBTripleArray;

////////////////////////////////////////////////////////////

procedure TLineProfileForm.FormCreate(Sender: TObject);
begin
  left := LineProfileFormX;
  top := LineProfileFormY;
  TabSheetGraphs.TabVisible := FALSE;
  TabSheetData.TabVisible := FALSE;

  ProfileDrawing := False;
  Profiling := FALSE;
  ProfileLineDrawing := FALSE;
  DrawingTool := dtLineTool;

  LabelPoints.Caption := '';
  LabelFilename.Caption := '';
end;

procedure TLineProfileForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  Profiling := False;
  ProfileDrawing := False;
  ProfileLineDrawing := False;
  FreeImage;
  Do_LoadedEnable(True);
  Do_ExtraEnable(True);
{  UndoImage;}
  LineProfileFormX := LineProfileForm.left;
  LineProfileFormY := LineProfileForm.top;
  DoSaver;
end;

procedure TLineProfileForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TLineProfileForm.HelpBtnClick(Sender: TObject);
begin
  Application.HelpContext(7000);
end;

procedure TLineProfileForm.StartProfile;
var
  s: string;
begin
  DrawingTool := dtLineTool;
    // Get rid of any previous line
  ULPStartPoint := Point(OffScreenCoordinate, OffScreenCoordinate);
  ULPEndPoint := ULPStartPoint;
    // Make sure this is fully qualified name.
    // This will be used to save
    // image .INI file with alignment points.
  s := ExtractFilename(Filename);
  LabelFilename.Caption := COPY(s, 1, LENGTH(s) - 4);
  UpdateImage;
  GroupBoxDraw.Visible := TRUE;
  Profiling := True;
end;
////////////////////////////////////////////////////////////
  // Draw End Point "Handle"

procedure DrawEndPoint(const Canvas: TCanvas;
  const Point: TPoint;
  const PenColor: TColor;
  const BrushColor: TColor;
  const HalfSize: INTEGER {Pixels});
begin
  with Canvas do
  begin
    Pen.Color := PenColor;
    Pen.Style := psSolid;
    Pen.Width := 1; {Hardwire for now}

    Brush.Color := BrushColor;

    Rectangle(Point.X - HalfSize, Point.Y - HalfSize,
      Point.X + HalfSize, Point.Y + HalfSize)
  end
end {DrawEndPoint};


function OKPoint(const p: TPoint): BOOLEAN;
begin
  RESULT := (p.X <> OffScreenCoordinate) and
    (p.Y <> OffScreenCoordinate)
end {OKPoint};


function Distance(const a, b: TPoint): DOUBLE;
begin
  RESULT := SQRT(SQR(a.X - b.X) + SQR(a.Y - b.Y))
end {Distance};


  // RGB, each 0 to 255, to HSV.
  //   H = 0 to 359 (corresponding to 0..359 degrees around hexcone)
  //   S = 0 (shade of gray) to 255 (pure color)
  //   V = 0 (black) to 255 {white)

  // Based on C Code in "Computer Graphics -- Principles and Practice,"
  // Foley et al, 1996, p. 592.
  // Floating point fractions, 0..1, replaced with
  // integer values, 0..255.
                            {r, g and b IN [0..255]}

procedure RGBtoHSV(const RGB: TRGBTriple;
  var H, S, V: INTEGER);
                      {h IN 0..359; s,v IN 0..255}
var
  Delta: INTEGER;
  MinValue: INTEGER;
begin
  with RGB do
  begin
    MinValue := MinIntValue([rgbtRed, rgbtGreen, rgbtBlue]);
    V := MaxIntValue([rgbtRed, rgbtGreen, rgbtBlue])
  end;

  Delta := V - MinValue;

    // Calculate saturation:  saturation is 0 if r, g and b are all 0
  if V = 0
    then S := 0
  else S := (255 * Delta) div V;

  if S = 0
    then H := 0
    {Achromatic:  When s = 0, h is undefined but assigned the value 0}
  else begin
     {Chromatic}

    with RGB do
    begin
      if rgbtRed = V
        then H := (60 * (rgbtGreen - rgbtBlue)) div Delta
        {degrees -- between yellow and magenta}
      else
        if rgbtGreen = V
          then H := 120 + (60 * (rgbtBlue - rgbtRed)) div Delta
          {degrees -- between cyan and yellow}
        else
          if rgbtBlue = V
            then H := 240 + (60 * (rgbtRed - rgbtGreen)) div Delta;
             {degrees -- between magenta and cyan}
    end;

    if H < 0
      then H := H + 360;

  end
end {RGBtoHSV};

///////////////////////////////////////////////////////////////

procedure TLineProfileForm.SpeedButtonEraseClick(Sender: TObject);
begin
  ULPStartPoint := Point(OffScreenCoordinate, OffScreenCoordinate);
  ULPEndPoint := Point(OffScreenCoordinate, OffScreenCoordinate);
  UpdateImage; {Really only erases the Graph}
{  UndoImage;}{So do this}
end;


procedure TLineProfileForm.SpeedButtonToolClick(Sender: TObject);
begin
  case (Sender as TSpeedButton).Tag of
    1: DrawingTool := dtLineTool;
    2: DrawingTool := dtSelectTool
  end;
end;



  // Usually UpdateImage is too sluggish to be called on MouseMove events

procedure TLineProfileForm.UpdateImage;
begin
  if OKPoint(ULPStartPoint) and OKPoint(ULPEndPoint)
    then begin
    with GRIPForm.Image1.Canvas do
    begin // Use bsClear so brush isn't background of dotted line
      Brush.Style := bsClear;
      Pen.Color := clRed;
      Pen.Style := psSolid;
      Pen.Width := 1;
      MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
      LineTo(ULPEndPoint.X, ULPEndPoint.Y);
      DrawEndPoint(GRIPForm.Image1.Canvas, ULPStartPoint,
        clBlack, clRed, SquareHalfSize);
      DrawEndPoint(GRIPForm.Image1.Canvas, ULPEndPoint,
        clBlack, clRed, SquareHalfSize)
    end
  end;
  UpdateLineLength;
  CreateProfileLine;
end {UpdateImages};


procedure TLineProfileForm.UpdateLineLength;
var
  s: string;
begin
  s := 'Points = ';

  s := s + '(' + IntToStr(ULPStartPoint.x) + ', ' +
    IntToStr(ULPStartPoint.y) + ')   ';
  s := s + '(' + IntToStr(ULPEndPoint.x) + ', ' +
    IntToStr(ULPEndPoint.y) + ')   ';

  if OKPoint(ULPStartPoint) and
    OKPoint(ULPEndPoint)
    then begin
    s := s + '  Distance = ' +
      Format('%.1f', [Distance(ULPStartPoint, ULPEndPoint)]);
    LabelPoints.Caption := s;

    ChartPixelProfile.Title.Text.Clear;
    ChartPixelProfile.Title.Text.Add(
      'Pixel Profile:   ' +
      LabelFilename.Caption +
      '   from (' +
      IntToStr(ULPStartPoint.x) + ', ' +
      IntToStr(ULPStartPoint.y) + ') to (' +
      IntToStr(ULPEndPoint.x) + ', ' +
      IntToStr(ULPEndPoint.y) + ')');
  end
  else LabelPoints.Caption := ''
end {UpdateLineLength};


procedure TLineProfileForm.CreateProfileLine;
var
  BitmapSlice: TBitmap;
  fraction: DOUBLE;
  H, S, V: INTEGER;
  i, j: DWord;
  Intensity: BYTE;
  itoo, k: INTEGER;
  m: INTEGER;
  PixelDistance: INTEGER;
  RGB: TRGBTriple;
  x, y: DOUBLE;
  PGreenA, PBlueA, PredA: array of Byte;
  xwhere: INTEGER;
  TempStringer, TempRString, TempGString, TempBString: string;
  PGreen, PBlue, Pred: DOUBLE;
begin
  if ImageMade = TRUE then begin
    Screen.Cursor := crHourGlass;

    try
      if OKPoint(ULPStartPoint) and OKPOint(ULPEndPoint)
        then
        PixelDistance := TRUNC(Distance(ULPStartPoint, ULPEndPoint) +
          0.5)
      else PixelDistance := 0;

      // Don't bother unless at least 2 pixels are in the profile line
      if PixelDistance < 2
        then begin
        ImagePixelRow.Picture := nil;
        TabSheetGraphs.TabVisible := FALSE;
        TabSheetData.TabVisible := FALSE;
      end
      else begin
        SetLength(PGreenA, PixelDistance + 1);
        SetLength(PBlueA, PixelDistance + 1);
        SetLength(PredA, PixelDistance + 1);
        TabSheetGraphs.TabVisible := TRUE;
        TabSheetData.TabVisible := TRUE;

        ChartPixelProfile.BottomAxis.Maximum := PixelDistance;

        case PixelDistance of
          0..10: ChartPixelProfile.BottomAxis.Increment := 1;
          11..50: ChartPixelProfile.BottomAxis.Increment := 5;
          51..125: ChartPixelProfile.BottomAxis.Increment := 10;
          126..250: ChartPixelProfile.BottomAxis.Increment := 25;
        else
          ChartPixelProfile.BottomAxis.Increment := 50;
        end;

        BitmapSlice := TBitmap.Create;
        try
          BitmapSlice.Height := 3;
          BitmapSlice.PixelFormat := pf24Bit;
          BitmapSlice.Width := PixelDistance + 1;

          SeriesRed.Clear;
          SeriesGreen.Clear;
          SeriesBlue.Clear;
          SeriesIntensity.Clear;
          SeriesHue.Clear;
          SeriesSaturation.Clear;
          SeriesValue.Clear;
          MemoData.Clear;
{FixedSys font to make the columns align
         Compute data into new columns}
          MemoData.Lines.Add(LabelFilename.Caption);
          MemoData.Lines.Add(' ');
          MemoData.Lines.Add(' #      X     Y      R   slope  G   slope  B   slope  I     H   S   V');
          MemoData.Lines.Add('---   ----- -----   --- ------ --- ------ --- ------ ---   --- --- ---');
                                             {24}
          for k := 0 to PixelDistance do
          begin
            fraction := k / PixelDistance;
            // Add 0.5 to endpoints so "line" is from center of pixel
            x := (1.0 - fraction) * (ULPStartPoint.X + 0.5) +
              fraction * (ULPEndPoint.X + 0.5);

            y := (1.0 - fraction) * (ULPStartPoint.Y + 0.5) +
              fraction * (ULPEndPoint.Y + 0.5);

            i := TRUNC(x);
            j := TRUNC(y);
 {Grab real data from Undo bitmap}{ImageHeight is already -1 }
            WherePixel := (3 * (i)) + (BytesPerLine * (((ImageHeight
              - j))));
            lpPixelB := @PHugeByteArray(Undo)^[WherePixel];
            lpPixelG := @PHugeByteArray(Undo)^[WherePixel + 1];
            lpPixelR := @PHugeByteArray(Undo)^[WherePixel + 2];
            RGB.rgbtBlue := lpPixelB^;
            RGB.rgbtGreen := lpPixelG^;
            RGB.rgbtRed := lpPixelR^;
            PGreenA[k] := lpPixelG^;
            PBlueA[k] := lpPixelB^;
            PredA[k] := lpPixelR^;
{ RGB := pRGBTripleArray(RasterForm.Image1.Picture.Bitmap.Scanline[j])[i];}

            for m := 0 to BitmapSlice.Height - 1 do
            begin
              pRGBTripleArray(BitmapSlice.Scanline[m])[k] := RGB
            end;

            RGBToHSV(RGB, H, S, V);

            Intensity := (RGB.rgbtred + RGB.rgbtgreen + RGB.rgbtblue)
              div 3;
            MemoData.Lines.Add(
              Format('%3d   %5.1f %5.1f   %3d        %3d        %3d        %3d   %3d %3d %3d',
              [k, x, y, RGB.rgbtred, RGB.rgbtgreen, RGB.rgbtblue,
              Intensity, H, S, V]));
            SeriesRed.AddXY(k, RGB.rgbtRed, '', clTeeColor);
            SeriesGreen.AddXY(k, RGB.rgbtGreen, '', clTeeColor);
            SeriesBlue.AddXY(k, RGB.rgbtBlue, '', clTeeColor);

            SeriesIntensity.AddXY(k, Intensity, '', clTeeColor);

            SeriesHue.AddXY(k, H, '', clTeeColor);
            SeriesSaturation.AddXY(k, S, '', clTeeColor);
            SeriesValue.AddXY(k, V, '', clTeeColor)
          end;
          ImagePixelRow.Picture.Graphic := BitmapSlice;
{Compute the Slopes}
          for xwhere := 3 to PixelDistance - 3 do begin
            Pred := ((((PredA[xwhere - 2]) + (PredA[xwhere - 1])
              + (PredA[xwhere]) + (PredA[xwhere + 1])
              + (PredA[xwhere + 2]))
              / 5)
              - (((PredA[xwhere - 2]) + (PredA[xwhere - 1])
              + (PredA[xwhere]) + (PredA[xwhere + 1])
              + (PredA[xwhere - 3]))
              / 5));
            PGreen := ((((PGreenA[xwhere - 2]) + (PGreenA[xwhere - 1])
              + (PGreenA[xwhere]) + (PGreenA[xwhere + 1])
              + (PGreenA[xwhere + 2]))
              / 5)
              - (((PGreenA[xwhere - 2]) + (PGreenA[xwhere - 1])
              + (PGreenA[xwhere]) + (PGreenA[xwhere + 1])
              + (PGreenA[xwhere - 3]))
              / 5));
            PBlue := ((((PBlueA[xwhere - 2]) + (PBlueA[xwhere - 1])
              + (PBlueA[xwhere]) + (PBlueA[xwhere + 1])
              + (PBlueA[xwhere + 2]))
              / 5)
              - (((PBlueA[xwhere - 2]) + (PBlueA[xwhere - 1])
              + (PBlueA[xwhere]) + (PBlueA[xwhere + 1])
              + (PBlueA[xwhere - 3]))
              / 5));
            str(Pred: 6: 1, TempRString);
            str(PGreen: 6: 1, TempGString);
            str(PBlue: 6: 1, TempBString);
            TempStringer := MemoData.Lines[xwhere + 4];
            for itoo := Length(TempRString) downto 1 do
              TempStringer[23 + itoo] := TempRString[itoo];
            for itoo := Length(TempGString) downto 1 do
              TempStringer[34 + itoo] := TempGString[itoo];
            for itoo := Length(TempBString) downto 1 do
              TempStringer[45 + itoo] := TempBString[itoo];

            MemoData.Lines[xwhere + 4] := TempStringer;
          end;
        finally
          BitmapSlice.Free;
          SetLength(PGreenA, 0);
          SetLength(PBlueA, 0);
          SetLength(PredA, 0);
        end
      end
    finally
      Screen.Cursor := crDefault
    end
  end else DoMessages(39975);
end {CreateProfileLine};


//////////////////////////////////////////////////////////////

procedure TLineProfileForm.DoImageMouseDown(X, Y: Integer);
{procedure TLineProfileForm.ImageMouseDown(Sender: TObject;
                        Button: TMouseButton;
                        Shift: TShiftState; X, Y: Integer);}
begin
  with GRIPForm.Image1.Canvas do
  begin
    Pen.Color := clRed;
    Pen.Style := psSolid;
    Pen.Width := 1;
    Pen.Mode := pmCopy
  end;
  case DrawingTool of
    dtLineTool:
      begin
        ProfileLineDrawing := TRUE;
        ULPStartPoint := Point(X, Y);
        ULPEndPoint := Point(X, Y);
        Screen.Cursor := crCross;
        RestrictCursorToDrawingArea(GRIPForm.Image1);
        DrawEndPoint(GRIPForm.Image1.Canvas, ULPStartPoint,
          clBlack, clRed, SquareHalfSize)
      end;
    dtSelectTool:
      begin
        Selected := lsNotSelected;
        MovingPoint := Point(X, Y);
        if SquareContainsPoint(ULPStartPoint,
          SquareHalfSize,
          MovingPoint)
          then Selected := lsPoint1
        else
          if SquareContainsPoint(ULPEndPoint,
            SquareHalfSize,
            MovingPoint)
            then Selected := lsPoint2
          else
            if NearLine(MovingPoint, ULPStartPoint, ULPEndPoint)
              then Selected := lsLine;

        if Selected in [lsPoint1, lsPoint2, lsLine]
          then begin
          Screen.Cursor := crDrag;
          RestrictCursorToDrawingArea(GRIPForm.Image1)
        end
      end
  end;

  with GRIPForm.Image1.Canvas do
  begin
    Pen.Style := psSolid;
    Pen.Mode := pmNotXOR;
    Pen.Color := RubberBandColor;
    Pen.Width := 1;
  end
end;

procedure TLineProfileForm.DoImageMouseMove(X, Y: Integer);
{procedure TLineProfileForm.ImageMouseMove(Sender: TObject;
                        Shift: TShiftState; X, Y: Integer);}
var
  Delta: TPoint;
begin
  // Ignore unless/until ProfileDrawing
  case DrawingTool of
    dtLineTool:
      begin
          // If drawing new line
        if ProfileLineDrawing
          then begin
          with GRIPForm.Image1.Canvas do
          begin
              // Use NOT XOR mode to erase old line
            MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
            LineTo(ULPEndPoint.X, ULPEndPoint.Y);

              // Draw new line
            ULPEndPoint := Point(X, Y);
            MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
            LineTo(ULPEndPoint.X, ULPEndPoint.Y);

    // Additional information displayed here, namely, X,Y, R,G,B and
    // intensity caused unacceptable flicker, so these were deleted.
            UpdateLineLength
          end
        end
      end;

    dtSelectTool:
      begin
        case Selected of
          lsPoint1:
            begin
              with GRIPForm.Image1.Canvas do
              begin
                  // Use NOT XOR mode to erase old line
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);

                  // Draw new line
                ULPStartPoint := Point(X, Y);
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);
              end;

              UpdateLineLength
            end;

          lsPoint2:
            begin
              with GRIPForm.Image1.Canvas do
              begin
                  // Use NOT XOR mode to erase old line
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);

                  // Draw new line
                ULPEndPoint := Point(X, Y);
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);
              end;

              UpdateLineLength
            end;

          lsLine:
            begin
              with GRIPForm.Image1.Canvas do
              begin
                  // Use NOT XOR mode to erase old line
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);

                Delta := SubtractPoints(Point(X, Y), MovingPoint);
                MovingPoint := Point(X, Y);
                ULPStartPoint := AddPoints(ULPStartPoint, Delta);
                ULPEndPoint := AddPoints(ULPEndPoint, Delta);
                MoveTo(ULPStartPoint.X, ULPStartPoint.Y);
                LineTo(ULPEndPoint.X, ULPEndPoint.Y);
              end
            end

        else
          if SquareContainsPoint(ULPStartPoint, SquareHalfSize,
            Point(X, Y))
            or
            SquareContainsPoint(ULPEndPoint, SquareHalfSize, Point(X,
            Y))
            or
            NearLine(Point(X, Y), ULPStartPoint, ULPEndPoint)
            then Screen.Cursor := crHandPoint
          else Screen.Cursor := crDefault
        end
      end
  end
end;


procedure TLineProfileForm.DoImageMouseUp(X, Y: Integer);
{procedure TLineProfileForm.ImageMouseUp(Sender: TObject;
            Button: TMouseButton;
            Shift: TShiftState; X, Y: Integer); }
var
  Delta: TPoint;
begin
  // Ignore unless/until bitmap is present
  case DrawingTool of
    dtLineTool:
      begin
        ULPEndPoint := Point(X, Y);
        UpdateImage;
        ProfileLineDrawing := FALSE;
        Screen.Cursor := crDefault;
        RemoveCursorRestrictions
      end;

    dtSelectTool:
      begin
              // Final adjustment
        case Selected of
          lsPoint1: ULPStartPoint := Point(X, Y);
          lsPoint2: ULPEndPoint := Point(X, Y);

          lsLine:
            begin
              Delta := SubtractPoints(Point(X, Y), MovingPoint);
              MovingPoint := Point(X, Y);
              ULPStartPoint := AddPoints(ULPStartPoint, Delta);
              ULPEndPoint := AddPoints(ULPEndPoint, Delta)
            end;
        end;

        if Selected <> lsNotSelected
          then begin
          UpdateImage;
          Selected := lsNotSelected;
          Screen.Cursor := crDefault;
          RemoveCursorRestrictions
        end;
      end;
  end;
end;
///////////////////////////////////////////////////////////

procedure TLineProfileForm.CheckBoxPlaneClick(Sender: TObject);
begin
  case (Sender as TCheckBox).Tag of
    1: SeriesRed.Active := CheckboxRed.Checked;
    2: SeriesGreen.Active := CheckboxGreen.Checked;
    3: SeriesBlue.Active := CheckboxBlue.Checked;

    4: SeriesIntensity.Active := CheckboxIntensity.Checked;

    5: SeriesHue.Active := CheckboxHue.Checked;
    6: SeriesSaturation.Active := CheckboxSaturation.Checked;
    7: SeriesValue.Active := CheckboxValue.Checked;
  else
      // Do nothing
  end;
end;

procedure TLineProfileForm.ButtonPrintGraphClick(Sender: TObject);
begin
  ChartPixelProfile.PrintLandScape
end;


procedure TLineProfileForm.efgLabelClick(Sender: TObject);
begin
  ShellExecute(0, 'open', pchar(efgLabel.Caption),
{  'http://www.efg2.com/Lab/',}'', '', SW_SHOW);
end;



////////////////////////////////////////////////////////////

procedure TLineProfileForm.SaveButtonClick(Sender: TObject);
var S: string;
begin
  s := ChangeFileExt(FilePath + FileName, '.txt');
  MemoData.Lines.SavetoFile(s);
end;

procedure TLineProfileForm.PrintButtonClick(Sender: TObject);
begin
  MemoData.Print(FileName {'My Document Name'});
end;

procedure TLineProfileForm.CopyButtonClick(Sender: TObject);
begin
  MemoData.SelectAll;
  MemoData.CopyToClipboard;
end;

end.
