unit szDupDlg;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  ComCtrls, ExtCtrls, StdCtrls, ClipBrd,
  {FormSettings,} Buttons{vcl};

type
  TszDupForm = class(TForm)
    TopPanel: TPanel;
    ListPanel: TPanel;
    FileHeader: THeaderControl;
    ListBox1: TListBox;
    StatusBar1: TStatusBar;
    CheckBut: TButton;
    DelBut: TButton;
    DelOtherBut: TButton;
    CopyBut: TButton;
    AllowMisBox: TCheckBox;
    btnInfo: TSpeedButton;
    btnHelp: TSpeedButton;
    procedure ListBox1DrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure CheckButClick(Sender: TObject);
    procedure FileHeaderSectionResize(HeaderControl: THeaderControl;
      Section: THeaderSection);
    procedure DelButClick(Sender: TObject);
    procedure FileHeaderSectionClick(HeaderControl: THeaderControl;
      Section: THeaderSection);
    procedure CopyButClick(Sender: TObject);
    procedure AllowMisBoxClick(Sender: TObject);
    procedure btnInfoClick(Sender: TObject);
    procedure btnHelpClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  protected
  public
    { Public declarations }
    procedure  CheckButs;
    function   Calc_CheckSums(Idx: integer; WarnUser: boolean): boolean;
  end;

var
  szDupForm: TszDupForm;

implementation

uses TongGlobals, szMain, szScanThread, szRecycle, szAdler, PicAbout;

{$R *.DFM}

procedure TszDupForm.FormCreate(Sender: TObject);
begin
  left := szDupFormX;
  top := szDupFormY;
end;

procedure TszDupForm.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  szDupFormX := szDupForm.left;
  szDupFormY := szDupForm.top;
end;

procedure TszDupForm.btnInfoClick(Sender: TObject);
begin
  PicAboutBox.Init(4);
  PicAboutBox.ShowModal;
end;

procedure TszDupForm.btnHelpClick(Sender: TObject);
begin
  Application.HelpContext(4700);
end;



procedure TszDupForm.ListBox1DrawItem(Control: TWinControl; Index: Integer;
  Rect: TRect; State: TOwnerDrawState);
const
   DT_DTEXT = DT_LEFT or DT_BOTTOM or DT_END_ELLIPSIS;
var
   s     : string;
   d, d2 : TFileInfo;
   R     : TRect;
   w1    : integer;
   Hgt   : integer;
   x, y  : integer;
   OldColor : TColor;
begin
   x := Rect.Left;
   y := Rect.Top;
   Hgt := Rect.Bottom - Rect.Top;

   with ListBox1.Canvas do begin
      FillRect(Rect);

      d  := szMainForm.DupList[Index];

      // Draw the Icon
      szMainForm.ImageList1.Draw(ListBox1.Canvas, x+2, y+0, 2 {File});

      // Draw the Name
      s  := d.Name;
      w1 := FileHeader.Sections[0].Width;
      R := Classes.Rect(x+2+16+2, y+2, x+w1-4, y+Hgt);
      DrawTextEx(Handle, PChar(s), -1, R, DT_DTEXT, nil);

      // Draw the Size
      s := Format('%1.0n', [d.Size]);
      w1 := w1 + FileHeader.Sections[1].Width;
      R := Classes.Rect(R.Right+2, y+2, x+w1-4, y+Hgt);
      DrawTextEx(Handle, PChar(s), -1, R, DT_DTEXT, nil);

      // Draw Modified
      s := FormatDateTime('mm/dd/yy hh:nn ampm', d.LastWrite);
      w1 := w1 + FileHeader.Sections[2].Width;
      R := Classes.Rect(R.Right+2, y+2, x+w1-4, y+Hgt);
      DrawTextEx(Handle, PChar(s), -1, R, DT_DTEXT, nil);

      // Draw the CheckSum
      w1 := w1 + FileHeader.Sections[3].Width;
      R := Classes.Rect(R.Right+2, y+2, x+w1-4, y+Hgt);
      OldColor := Font.Color;
      s := Format('%4.4x-%4.4x', [d.CheckSum shr 16, d.CheckSum and $FFFF]);
      if (not d.ChkMatch) and (d.HasChkSum) then begin
         szMainForm.ImageList1.Draw(ListBox1.Canvas,
            R.Left + TextWidth(s) + 4, y, 3);
         Font.Color := clRed;
      end;
      if d.HasChkSum then begin
         DrawTextEx(Handle, PChar(s), -1, R, DT_DTEXT, nil);
      end;
      Font.Color := OldColor;

      // Draw the Path
      s := ExtractFilePath(d.FullPath);
      s := Copy(s, 1, Length(s)-1);
      w1 := w1 + FileHeader.Sections[4].Width;
      R := Classes.Rect(R.Right+2, y+2, x+w1-4, y+Hgt);
      DrawTextEx(Handle, PChar(s), -1, R, DT_LEFT or DT_BOTTOM or DT_PATH_ELLIPSIS, nil);

      // Draw the Dividing Line
      if (Index < szMainForm.DupList.Count-1) then begin
         d2 := szMainForm.DupList[Index + 1];
         if CompareDupFiles(d, d2, False, not AllowMisBox.Checked) <> 0 then begin
            Pen.Style := psDot;
            MoveTo(Rect.Left, Rect.Bottom-1);
            LineTo(Rect.Right, Rect.Bottom-1);
            Pen.Style := psSolid;
         end;
      end;

   end;
end;

procedure TszDupForm.FormResize(Sender: TObject);
begin
   ListBox1.Invalidate;
end;

procedure TszDupForm.CheckButs;
var
   b : boolean;
begin
   b := ListBox1.ItemIndex <> -1;
   CheckBut.Enabled     := b;
   DelBut.Enabled       := b;
   DelOtherBut.Enabled  := b;
end;

procedure TszDupForm.FormShow(Sender: TObject);
begin
   CheckButs;
end;

procedure TszDupForm.ListBox1Click(Sender: TObject);
begin
   CheckButs;
end;

function TszDupForm.Calc_CheckSums(Idx: integer; WarnUser: boolean): boolean;
var
   Min, Max, i : integer;
   fi          : TFileInfo;
   Match       : boolean;
begin
   Assert(Idx<>-1);
   {$B-}
   Screen.Cursor := crHourGlass;
   try
      with szMainForm.DupList do begin
         Min := Idx;
         while (Min > 0) and (CompareDupFiles(Items[Min], Items[Min-1], False, not AllowMisBox.Checked) = 0) do
            Min := Min - 1;

         Max := Idx;
         while (Max < Count-1) and (CompareDupFiles(Items[Max], Items[Max+1], False, not AllowMisBox.Checked) = 0) do
            Max := Max + 1;

         for i := Min to Max do begin
            fi           := Items[i];
            fi.CheckSum  := CheckSumFile(fi.FullPath);
            fi.HasChkSum := True;
            fi.ChkMatch  := True;
            ListBox1.Refresh;
         end;

         Match := True;
         fi := Items[Min];
         for i := Min+1 to Max do begin
            if TFileInfo(Items[i]).CheckSum <> fi.CheckSum then
               Match := False;
         end;

         for i := Min+1 to Max do begin
            TFileInfo(Items[i]).ChkMatch := Match;
         end;

         ListBox1.Invalidate;
         if WarnUser and (not Match) then begin
            MessageDlg('Checksums do NOT MATCH!', mtWarning, [mbOk], 0);
         end;
      end;
   finally
      Screen.Cursor := crDefault;
   end;
   Result := Match;
end;

procedure TszDupForm.CheckButClick(Sender: TObject);
var
   i     : integer;
   Match : boolean;
begin
   if ListBox1.SelCount > 0 then begin
      Match := True;
      for i := 0 to ListBox1.Items.Count-1 do
         if ListBox1.Selected[i] then
            Match := Calc_CheckSums(i, False) and Match;    // watch out for short-circuit boolean evaluation!

      if (not Match) then
         MessageDlg('Checksums do NOT MATCH!', mtWarning, [mbOk], 0);
   end;
end;

procedure TszDupForm.FileHeaderSectionResize(HeaderControl: THeaderControl;
  Section: THeaderSection);
begin
   ListBox1.Invalidate;
end;

procedure TszDupForm.DelButClick(Sender: TObject);
var
   i  : integer;
   fi : TFileInfo;
   s  : string;
begin
   CheckButClick(nil);

   s := '';
   for i := 0 to ListBox1.Items.Count-1 do begin
      fi := szMainForm.DupList[i];
      if (ListBox1.Selected[i]) then begin
         if (fi.ChkMatch) then
            s := s + fi.FullPath + #0
         else begin
            MessageDlg('This file is not a true duplicate: ' + fi.FullPath + #13#10 +
               '(You may still delete this file from the main form as usual)', mtError, [mbOk], 0);
            exit;
         end;
      end;
   end;

   if RecycleFile(s) then begin
      for i := ListBox1.Items.Count-1 downto 0 do begin
         fi := szMainForm.DupList[i];
         if ListBox1.Selected[i] and (not FileExists(fi.FullPath)) then begin
            szMainForm.DupList.Delete(i);
            ListBox1.Items.Delete(i);
         end;
      end;
      CheckButs;
   end;
end;

function SortName(Item1, Item2: Pointer): Integer;
begin
   Result := CompareDupFiles_Name(Item1, Item2, True);
end;

function SortSize(Item1, Item2: Pointer): Integer;
begin
   Result := CompareDupFiles(Item1, Item2, True, True);
end;

procedure TszDupForm.FileHeaderSectionClick(HeaderControl: THeaderControl;
  Section: THeaderSection);
var
   i  : integer;
   fi : TFileInfo;
begin
   if (Section.ID<>0) and (Section.ID<>1) then begin
      MessageDlg('You may only sort by Name or Size', mtError, [mbOk], 0);
      exit;
   end;

   i := ListBox1.ItemIndex;
   fi := nil;
   if i<>-1 then fi := szMainForm.DupList[i];

   if Section.ID=0 then
      szMainForm.DupList.Sort(SortName);
   if Section.ID=1 then
      szMainForm.DupList.Sort(SortSize);

   ListBox1.Invalidate;
   ListBox1.ItemIndex := szMainForm.DupList.IndexOf(fi);
end;

procedure TszDupForm.CopyButClick(Sender: TObject);
var
   i        : integer;
   fi       : TFileInfo;
   GotOne   : boolean;
   buf, s   : string;
   dt, ck   : string;
begin
   GotOne := False;
   buf := '';

   for i := ListBox1.Items.Count-1 downto 0 do begin
      if not ListBox1.Selected[i] then continue;
      GotOne := True;

      fi := szMainForm.DupList[i];
      dt := FormatDateTime('mm/dd/yy hh:nn ampm', fi.LastWrite);
      if fi.HasChkSum then
         ck := Format('%4.4x-%4.4x', [fi.CheckSum shr 16, fi.CheckSum and $FFFF])
      else
         ck := ' ';
      s := Format('%s'#9'%1.0n'#9'%s'#9'%s'#9'%s', [fi.Name, fi.Size, dt, ck, fi.FullPath]);
      buf := buf + s + #13#10;
   end;

   if not GotOne then begin
    MessageDlg('Nothing Selected to copy to the Clipboard!'#13#10+
         '(You must select items in the list below)', mtWarning, [mbOk], 0);
   end else begin
      Clipboard.AsText := buf;
   end;
end;

procedure TszDupForm.AllowMisBoxClick(Sender: TObject);
begin
   ListBox1.Invalidate;
end;


end.
